public class SearchTree<T extends Comparable<T>> {

    private class Node {
	T    value;
        Node left;
	Node right;

	Node(T val) {
	    value = val;
	    left = null;
	    right = null;
	}
    }

    private Node root;
    private int size;

    public SearchTree() {
        root = null;
	size = 0;
    }

    public int size() {
        return size;
    }

    public int height() {
        return height(root);
    }

    private int height(Node node) {
        if (node==null) {		// Der leere Baum hat die Hoehe 0
	    return 0;
	}
	else {				// ansonsten h(T) = 1+max{h(T_l), h(T_r)}
	    return 1+Math.max(height(node.left),height(node.right));
	}
    }

    public void sortedArray(T[] elems) {
	int index;

        if (elems.length < size) {			// Exception, falls Feld nicht gross genug
	    throw new IllegalArgumentException();
	}

	fillArray(root,elems,0);	// schreibe vollstaendigen Baum (Wurzel) nach Feld ab Feldanfang
    }

    /*
     * Kopiere Objekte des Unterbaums mit Wurzel node in Sortierreihenfolge (inorder)
     * nach Feld elemes ab Stelle index und
     * liefere als Ergebnis die Gesamtanzahl der bisher kopierten Elemente.
     */
    private int fillArray(Node node, T[] elems, int index) {
        if(node!=null) {				// nur wenn Unterbaum nicht leer
	    index = fillArray(node.left,elems,index);   // kopiere linken Unterbaum ins Feld
	    elems[index] = node.value;                  // kopiere Wert am Knoten in Feld und
	    index++;                                    // kopiere rechten Unterbaum ins Feld.
	    index = fillArray(node.right,elems,index);
        }

	return index;
    }

    public boolean contains(T elem) {
        Node node = root;
        int  diff;

	while (node != null) {
	    diff = node.value.compareTo(elem);
	    if (diff == 0) {
	        return true;
	    }
	    else if (diff>0) {
	        node = node.left;
	    }
	    else {
	        node = node.right;
	    }
	}

	return false;
    }

    public boolean containsR(T elem) {			// Implementierung durch private
        return containsR(root,elem);			// rekursive Methode containsR()
    }

    private boolean containsR(Node node, T elem) {	// Ist elem in Unterbaum mit Wurzel node?
	int diff;					// Ergebnis des Vergleichs zwischen
							// node.value und elem
        if (node == null) {				// Unterbaum leer,
	    return false;				// dann nicht enthalten
	}

	diff = node.value.compareTo(elem);		// vergleiche node.value und elem
	if (diff == 0) {				// wenn ==
	    return true;				// dann gefunden
	}						// ansonsten
	else if (diff > 0) {				// wenn node.value > elem
	    return containsR(node.left,elem);		// suche im linken Unterbaum weiter
	}					
	else {						// wenn node.value < elem
	    return containsR(node.right,elem);		// suche im rechten Unterbaum weiter
	}
    }

    public void insert(T elem) {
        Node node = root;
	Node father = null;
	int  diff=0;

	while (node!=null) {
	    father = node;
	    diff = node.value.compareTo(elem);
	    if (diff==0) {
	        return;
	    }
	    else if (diff>0) {
	        node = node.left;
	    }
	    else {
	        node = node.right;
	    }
	}

	node = new Node(elem);

	if (father==null) {
	   root = node;
	}
	else if (diff>0) {
	    father.left = node;
	}
	else {
	    father.right = node;
	}

	size++;
    }

    public void remove(T elem) {
        Node node = root;
	Node father = null;
	Node node2;
	int  diff;

	if (root==null) {	// Baum ist leer
	    return;
	}

	do {					// Suche den zu loeschenden Knoten und bestimme
	    diff = node.value.compareTo(elem);	// seinen Vater
	    if (diff>0) {
	        father = node;
	        node = node.left;
	    }
	    else if (diff <0) {
	        father = node;
	        node = node.right;
	    }
	} while (diff!=0 && node!=null);

	if (node==null) {
	    return;		// Wert nicht gefunden
	}

	if (node.left==null || node.right==null) {	// Fall (a), (b 1) oder (b 2)
	    deleteNodeWithEmptySubtree(father, node);
	}
	else {						// Fall (c)
	    father = node;				
	    node2 = node.left;				// gehe zum linken sohn
	    while (node2.right!=null) {			// und ab dort so lange nach rechts wie moeglich 
	        father=node2;
		node2 = node2.right;
	    }						
	    node.value = node2.value;			// wert von node2 nach node kopieren und
	    deleteNodeWithEmptySubtree(father, node2);	// node2 mit Vater father loeschen
	}

	size--;
    }

    private void deleteNodeWithEmptySubtree(Node father, Node node) {
        Node subtree;					// Sohn von node, der fuer (b 1) oder (b 2)
	                                                // Sohn von father wird
	if (node.left==null && node.right==null) {	// Fall (a): node ist ein Blatt
	    subtree = null;				// father bekommt leeren Unterbaum als Sohn
	}
        else if (node.right==null) {			// Fall (b 1)
	    subtree = node.left;			// linker Unterbaum von node wird Sohn von father
	}
	else {						// Fall (b 2)
	    subtree = node.right;			// rechter Unterbaum von node wird Sohn von father
	}

        if (father==null) {				// Wird die Wuzel des gesamten Baumes geloescht?
	    root = subtree;				// Wenn die Wurzel ein Blatt war, ist der Baum jetzt leer
	}
	else if (node==father.left) {			// Wird subtree linker oder rechter Sohn?
	    father.left = subtree;
	}
	else {
	    father.right = subtree;
	}
    }

    public void printTree() {
        printTree(root, 0);
    }

    private void printTree(Node node, int level) {
        String s;
        if (node==null) {
	    return;
	}

        printTree(node.right,level+1);

	s = "";
	for (int i=0 ; i<level ; i++) {
	    s+= "   ";
	}
	System.out.println(s+node.value);

        printTree(node.left,level+1);
    }
}
